#!/usr/bin/env python

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import time

import events
import slist

class playlistitem:
    def __init__(self, song):
        self.song = song
        self.played = 0

    def __repr__(self):
        return `self.song`

    def __getattr__(self, attr):
        return self.song.__dict__[attr]

    def getname(self, songdb):
        return self.song.getname(songdb)

    def play(self, player):
        self.played = 1
        self.song.play()
        events.hub.notify(events.nextsong(player, self.song))

    def hasbeenplayed(self):
        return self.played


class playlist(slist.slist):
    def __init__(self, win, songdb, logfilename=None):
        self.logfilename = logfilename
        self.songdb = songdb
        self.ttime = 0
        self.ptime = 0
        
        slist.slist.__init__(self, win)
        
        self.win.channel.subscribe(events.playlistaddsong, self.playlistaddsong)
        self.win.channel.subscribe(events.playlistclear, self.playlistclear)
        self.win.channel.subscribe(events.playlistdeleteplayedsongs,
                                   self.playlistdeleteplayedsongs)
        self.win.channel.subscribe(events.playlistload, self.playlistload)
        self.win.channel.subscribe(events.playlistsave, self.playlistsave)
        self.win.channel.subscribe(events.requestnextsong, self.requestnextsong)
        

    def append(self, item):
        self.ttime += item.length
        slist.slist.append(self, item)

    def __delitem__(self, index):
        item = self[index]
        self.ttime -= item.length
        if item.hasbeenplayed():
            self.ptime -= item.length
        slist.slist.__delitem__(self, index)

    def clear(self):
        self.ttime = 0
        self.ptime = 0
        slist.slist.clear(self)
    
    def getselected(self):
        playlistitem = slist.slist.getselected(self)
        if playlistitem:
            return playlistitem.song
        else:
            return None

    def searchnextsong(self):
        for i in range(len(self)):
            if not self[i].hasbeenplayed():
                return self[i]
        return None

    def rateselection(self, rating):
        assert 0<=rating<=5, "only ratings between 0 and 5 are allowed"
        
        song = self.getselected()
        song.rating = rating
        self.songdb.updatesong(song)

        events.hub.notify(events.selectionchanged(self.getselected()))

    # statusbar input handler
    
    def saveplaylisthandler(self, name, key):
        name = name.strip()
        if key==ord("\n") and name!="":
            if name[-4:]!=".m3u":
                name = name + ".m3u"
            try:
                file = open(name, "w")
                for song in self:
                    file.write("%s\n" % song.path)
                file.close()
                self.songdb.registerplaylist(name)
            except (IOError, OSError):
                pass

    def loadplaylisthandler(self, name, key):
        if key==ord("\n"):
            if name[-4:]!=".m3u":
                name = name + ".m3u"
            try:
                file = open(name, "r")
                self.clear()
            
                for line in file.xreadlines():
                    if not line.startswith("#"):
                        self.append(playlistitem(self.songdb.getsong(line.strip())))

                file.close()
                self.win.update()
            except (IOError, OSError):
                pass

    # event handlers

    def playlistaddsong(self, event):
        self.append(playlistitem(event.song))
        self.win.update()

    def playlistclear(self, event):
        self.clear()
        self.win.update()
        
    def playlistdeleteplayedsongs(self, event):
        for i in range(len(self)-1,-1,-1):
            if self[i].hasbeenplayed():
                del self[i]
        self.win.update()

    def playlistsave(self, event):
        events.hub.notify(events.requestinput(_("Save playlist"),
                                              _("Name:"),
                                             self.saveplaylisthandler))

    def playlistload(self, event):
        events.hub.notify(events.requestinput(_("Load playlist"),
                                              _("Name:"),
                                             self.loadplaylisthandler))

    def requestnextsong(self, event):
        nextsong = self.searchnextsong()
        if nextsong:
            self.ptime += nextsong.length
            nextsong.play(event.player)
            if self.logfilename:
                logfile = open(self.logfilename, "a")
                logfile.write("%s: %s\n" % (time.asctime(), nextsong.path))
                logfile.close()
            self.win.update()
        else:
            events.hub.notify(events.nextsong(event.player, None))



        



